@@ -60,8 +60,7 @@ with suppress(ModuleNotFoundError, ImportError):
             df = df.set_index('name')
             df['language'] = [x[0] for x in df['languages']]
             df['gender'] = [
-                'male' if x == 'VoiceGenderMale' else 'female'
-                for x in df['gender']
+                'male' if x == 'VoiceGenderMale' else 'female' for x in df['gender']
             ]
             del df['languages']
             return df
@@ -118,6 +117,7 @@ with suppress(ModuleNotFoundError, ImportError):
             engine_kwargs: Stuff you don't need to worry about
             time_format: In what format the time (or even date) is transformed to text that is read.
                 See https://www.programiz.com/python-programming/datetime/strftime
+            verbose: Print the time every loop
 
         Returns:
             Nothing, just loops and says the time regularly
